from flask import Flask, render_template
from flask.ext.fas import FAS
from flask.ext.sqlalchemy import SQLAlchemy
import logging
import os

# the version as used in setup.py and docs
__version__ = "0.2.6"

logging.basicConfig(level=logging.INFO,
                    format='[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s',
                    datefmt='%Y%m%d-%H:%M%p')

# Flask App
app = Flask(__name__)


# load default configuration
if os.getenv('DEV') == 'true':
    app.config.from_object('blockerbugs.config.DevelopmentConfig')
elif os.getenv('TEST') == 'true':
    app.config.from_object('blockerbugs.config.TestingConfig')
else:
    app.config.from_object('blockerbugs.config.ProductionConfig')


# load real configuration values to override defaults
config_file = '/etc/blockerbugs/settings.py'
if not app.testing:
    if os.path.exists(config_file):
        app.logger.info('Loading configuration from %s' % config_file)
        app.config.from_pyfile(config_file)
    else:
        config_file = os.path.abspath('conf/settings.py')
        if os.path.exists(config_file):
            app.logger.info('Loading configuration from %s' % config_file)
            app.config.from_pyfile(config_file)
        else:
            app.logger.info('No extra config found, using defaults')

# setup logging
if app.debug:
    logging.basicConfig(level=logging.DEBUG,
                       format='[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s',
                       datefmt='%Y%m%d-%H:%M%p')


app.logger.debug('using DBURI: %s' % app.config['SQLALCHEMY_DATABASE_URI'])

# database 
db = SQLAlchemy(app)

# setup FAS extension
fas = FAS(app)


@app.template_filter('tagify')
def tagify(value):
    return value.replace(' ', '')

@app.template_filter('urlsplit')
def urlsplit(value):
    return value.split('/')

@app.template_filter('getname')
def getname(value, entry=1):
    return value.split('/')[entry]

@app.template_filter('getsubname')
def getsubname(value):
    urlparts = value.split('/')
    if len(urlparts) >= 5:
        return ''.join(urlparts[2:4])
    else:
        return ''

@app.template_filter('updatetype')
def updatetype(update):
    is_nth = False
    for bug in update.bugs:
        if bug.proposed_blocker or bug.accepted_blocker:
            return 'blocker'
        elif bug.proposed_nth or bug.accepted_nth:
            is_nth = True
    if is_nth:
        return 'NTH'

@app.template_filter('updatelabel')
def updatelabel(bug):
    label = []
    lastupdate = bug.updates.first()
    if lastupdate:
        if lastupdate.status == 'stable':
            if bug.status in ['MODIFIED', 'ON_QA', 'VERIFIED', 'CLOSED']:
                label.append('<span class="radius success label">')
                if lastupdate.pending:
                    label.append('pending ')
                label.append('stable')

        elif lastupdate.status == 'testing':
                label.append('<span class="radius alert label">')
                if lastupdate.pending:
                    label.append('pending ')
                label.append('testing')

    return ''.join(label)


# register blueprints
from blockerbugs.controllers.main import main
app.register_blueprint(main)

from blockerbugs.controllers.admin import admin
app.register_blueprint(admin)

from blockerbugs.controllers.users import users
app.register_blueprint(users)

# setup error handling
@app.errorhandler(404)
def page_not_found(e):
    return render_template('404.html', title="Page Not Found!")
