#
# update_sync.py - sync update information with bodhi
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>


from datetime import datetime
from lxml import etree, html
from fedora.client import bodhi
import logging
from urllib2 import urlopen

from blockerbugs.models.update import Update
from blockerbugs.models.bug import Bug

bodhi_baseurl = 'https://admin.fedoraproject.org/updates/'
bodhi_baserss = '%srss/rss2.0?' % bodhi_baseurl

class UpdateSync(object):
    def __init__(self, db, bodhi_interface=None):
        self.db = db
        if not bodhi_interface:
            self.bodhi_interface = bodhi.BodhiClient()
        else:
            self.bodhi_interface = bodhi_interface
        self.log = logging.getLogger('update_sync')

    def extract_information(self, update):
        updateinfo = {}
        if update.date_pushed:
            date_pushed = datetime.strptime(update.date_pushed, '%Y-%m-%d %H:%M:%S')
        updateinfo['date_pushed_testing'] = None
        updateinfo['date_pushed_stable'] = None
        if update.status == 'pending':
            updateinfo['pending'] = True

            if update.request == 'stable':
                updateinfo['status'] = 'stable'
                updateinfo['date_pushed_testing'] = date_pushed
            elif update.request == 'testing':
                updateinfo['status'] = 'testing'
            else:
                updateinfo['status'] = 'undefined'
        else:
            updateinfo['status'] = str(update.status)
            if update.status == 'testing':
                updateinfo['date_pushed_testing'] = date_pushed
            elif update.status == 'stable':
                updateinfo['date_pushed_stable'] = date_pushed


        updateinfo['title'] = str(update.title)
        updateinfo['karma'] = update.karma
        updateinfo['stable_karma'] = update.stable_karma

        updateinfo['url'] = '%s%s' % (bodhi_baseurl, updateinfo['title'])
        updateinfo['date_submitted'] = datetime.strptime(update.date_submitted, '%Y-%m-%d %H:%M:%S')

        updateinfo['bugs'] = []
        if len(update.bugs) > 0:
            for buginfo in update.bugs:
                updateinfo['bugs'].append(buginfo.bz_id)

        return updateinfo

    def get_rss_tree(self, release_num):
        rss_url = '%srelease=F%s' % (bodhi_baserss, release_num)
        parser = etree.XMLParser()
        return etree.parse(urlopen(rss_url), parser = parser)

    def get_update(self, envr):
        updates = self.bodhi_interface.query(package=envr)
        return self.extract_information(updates['updates'][0])

    def is_watched_bug(self, bugnum):
        watched_bug = Bug.query.filter_by(bugid=bugnum).first()
        if watched_bug:
            return True
        else:
            return False

    def get_fixing_updates(self, release):
        rssdoc = self.get_rss_tree(release.number)

        fixing_updates = []
        items = rssdoc.findall('channel/item')
        for item in items:
            # eventually, this should probably be a separate date from last_update
            # since ideally, bug and update sync should be separate
            pubdate = datetime.strptime(item.findtext('pubDate'), '%a, %d %b %Y %H:%M:%S %Z')
            if pubdate > release.last_update_sync:
                desc_frags = html.fragments_fromstring(item.findtext('description'))
                for frag in desc_frags:
                    if isinstance(frag, html.HtmlElement) and frag.tag == 'a':
                        bugid = int(frag.text)
                        if self.is_watched_bug(bugid):
                            update_title = item.findtext('title').split(' ')[-1]
                            fixing_updates.append(update_title)
        return fixing_updates


    def sync_updates(self, release):
        self.log.info('Syncing updates for Fedora %d with Bodhi RSS' % release.number)
        fixing_updates = self.get_fixing_updates(release)
        self.log.info('Found %d updates needing sync' % len(fixing_updates))

        for title in fixing_updates:
            update = self.get_update(title)
            existing_update = Update.query.filter_by(title=title).first()
            if existing_update:
                self.log.info('Updating record for %s' % update['title'])
                existing_update.sync(update)
            else:
                self.log.info('Adding new update for %s' % update['title'])
                existing_update = Update.from_data(update, release)
            self.db.session.add(existing_update)
            self.db.session.commit()

        # update the release's last sync time
        release.last_update_sync = datetime.utcnow()
        self.db.session.add(release)
        self.db.session.commit()

    def search_updates(self, bugids, release_num):
        updates = self.bodhi_interface.query(bugs=bugids, release='f%d' % release_num)
        self.log.info('found %d updates from bodhi' % len(updates['updates']))
        updateinfos = []
        for update in updates['updates']:
            updateinfos.append(self.extract_information(update))
        return updateinfos

    def get_release_bugs(self, release):
        buglist = []
        for milestone in release.milestones:
            buglist.extend(milestone.bugs.filter_by(active=True).all())
        return buglist

    def sync_bug_updates(self, release, bugs):
        for bug in bugs:
            self.log.info('searching for updates for bug %d' % bug.bugid)
            updates = self.search_updates([bug.bugid], release.number)
            self.log.info('found %d updateinfos for bug %d' % (len(updates), bug.bugid))
            for update in updates:
                self.log.info('running sync for update %s' % update['title'])
                existing_update = Update.query.filter_by(title=update['title']).first()
                if existing_update:
                    self.log.info('syncing existing update %s' % existing_update.title)
                    existing_update.sync(update, bug.milestone)
                else:
                    self.log.info('creating new update %s' % update['title'])
                    existing_update = Update.from_data(update, release, bug.milestone)
                self.db.session.add(existing_update)
                self.db.session.commit()

        # update the release's last sync time
        release.last_update_sync = datetime.utcnow()
        self.db.session.add(release)
        self.db.session.commit()

    def full_sync_updates(self, release):
        bugs = self.get_release_bugs(release)
        self.log.info('found %d bugs in current release' % len(bugs))
        self.sync_bug_updates(release, bugs)
